#!/bin/env python

from pop import POP # http://starship.python.net/crew/pson/pop.html
import re, smtplib, glob, os

import mimetypes
import email
from email import Encoders
from email.Message import Message
from email.MIMEAudio import MIMEAudio
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.MIMEText import MIMEText


DEBUG=True
def debug(message):
    if DEBUG:
        print message


SENDER_ADDRESS='BlueScreen <bluescreen@free.fr>'

class MailerInterpreter:


    def __init__(self, mail_server,login,passwd):
	debug("Connecting...")
	self.pop = POP(mail_server)
	self.pop.user(login,passwd)

        ## set defaults 
        self.share_path = "./"

        debug("OK, ready.")

    def set_config(self, configDict):
        if type(configDict) is dict:
            self.config = configDict

    def set_share_path(self,aPath):
        self.share_path = aPath
        if aPath[len(aPath)-1] != "/":
            self.share_path += "/"
        
    def dispatch_subjects(self):
	debug("Dispatching actions:")
	for id in range(1, len(self.pop)+1) :
            message_performed = False
	    subject_line = self.get_header("Subject:",id)
	    if subject_line is None:
		continue
	    if re.match(r"^Subject:.*##help##.*",subject_line):
	      message_performed = self.send_help(id)
	    if re.match(r"^Subject:.*##list##.*",subject_line):
	      message_performed = self.send_list(id)
	    if re.match(r"^Subject:.*##put##.*",subject_line):
	      message_performed = self.send_file(id)
	    if re.match(r"^Subject:.*##get##.*",subject_line):
	      message_performed = self.acquire_file(id)

            if message_performed:
                self.delete_mail(id)
            else:
                print "-%03d-> ERROR [%s]" % (id,subject_line)
                #self.delete_mail(id)


	debug("Done.")

    def get_header(self, header_name, mail_id):
	listOfHeaderLines = self.pop.top(mail_id)
	for line in listOfHeaderLines:
	    if line.find(header_name) != -1:
		return line
	return None

    def split_header(self,header_line):
	separator = ": "
	separator_index = header_line.find(separator)
	if separator_index == -1:
	    return None
	return (header_line[:separator_index],
                header_line[separator_index+len(separator):])
    
    def extract_from_adress(self,mail_id):
	from_header = self.get_header("From:",mail_id)
	if from_header is None:
	    print "No From in header"
	    return None
	(from_header,from_address) = self.split_header(from_header)
        return from_address
    
    #def __del__(self):
    #    self.quit()

    def quit(self):
	self.pop.quit()


    def send_help(self,mail_id):
        help_message = """<html><body>
<h1>------ MAIL SERVER USAGE ------</h1>
if you send a mail with a keyword in the subject, the server acts and sends you a reply.
<table style="border-collapse: collapse; border: 1px solid gray;">
 <tr>
  <th>Keywork in subject</th><th>the server</th>
 </tr>
 <tr>
  <td align="center"><pre>##help##</pre></td><td>sends you this message</td>
 </tr>
  <td align="center"><pre>##list##</pre></td><td>sends you the list of available files</td>
 </tr>
 <tr>
  <td align="center"><pre>##get##</pre></td><td>gets the attached file in your mail ("get from mail")</td>
 </tr>
  <td align="center"><pre>##put##</pre></td><td>sends you the file you requested ("put in mail")</td>
 </tr>
</table>
</body>
</html>
"""
        from_adress = self.extract_from_adress(mail_id)
        print "-%03d-> Sending help to %s" % (mail_id,from_adress)
        self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       ">>help requested", # subject
                       help_message,
                       attachedFile = None,
                       messageFormat='html') # mode
        return True

    def send_list(self,mail_id):
        fileList = glob.glob(self.share_path + "*.*")
        list_message = ""
        for fileName in fileList:
            shortFilename = os.path.split(fileName)[1]
            list_message += "<%s> %s\n" % (shortFilename, os.path.getsize(fileName) )
        
        from_adress = self.extract_from_adress(mail_id)
        subject = ">>list requested: %s files" % len(fileList)
        print "-%03d-> Sending list to %s" % (mail_id,from_adress)
        self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       subject, # subject
                       list_message)
        return True

    def send_file(self,mail_id):
        from_adress = self.extract_from_adress(mail_id)
        fullSubject = self.get_header("Subject",mail_id)
        filename = fullSubject[fullSubject.rfind("##")+2:].strip()
        if len(filename) == 0:
            return True
        
        print "-%03d-> Sending file <%s> to %s" % (mail_id,filename,from_adress)
        self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       ">>file requested", # subject
                       "File <%s>" % filename,# message
                       filename            # attached file
                       )

        return True

    def acquire_file(self,mail_id):
        from_adress = self.extract_from_adress(mail_id)
        print "-%03d-> Acquiring file from %s" % (mail_id,from_adress)
        messageString = "\n".join(self.pop[mail_id])
        mainMessage = email.message_from_string(messageString)
        allFiles = []
        for attachedMessage in mainMessage.walk():
            if attachedMessage.is_multipart():
                continue
            filename = attachedMessage.get_filename()
            if filename is None:
                continue
            file = open(self.share_path + filename,'wb')
            file.write(attachedMessage.get_payload(decode=True))
            file.close()
            allFiles.append(filename)
        self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       ">>get requested", # subject
                       "Done for %s" % "<br> - ".join(allFiles) # message
                       )
        return True

    def delete_mail(self,mail_id):
        del self.pop[mail_id]
	debug("  Message %03d deleted." % mail_id)
        
        

    def send_mail(self,from_address,to_address,subject,message,attachedFile = None,messageFormat = 'text'):
        outer = ""
        if attachedFile is None:
            outer = MIMEBase('multipart','mixed')
            outer.attach(MIMEText(message,messageFormat))
            #outer = MIMEText(message)
        else:
            outer = MIMEMultipart()
            outer.attach(MIMEText(message))
            outer.preamble = 'You will not see this in a MIME-aware mail reader.\n'
            outer.epilogue = ''
            self.__attach_file(self.share_path+attachedFile, outer, attachedFile)

        outer['Subject'] = subject
        outer['To'] = to_address
        outer['From'] = from_address
        server = smtplib.SMTP('localhost')
        server.sendmail(from_address,
                        [to_address],
                        outer.as_string()
                        #"""From: %s\nTo: %s\nSubject: %s\n\n%s""" %
                        #  (from_address,to_address,subject,message)
                        )
        server.quit()

    ##----------------------------------------------
    ## PRIVATE METHODS
    def __attach_file(self, path, outer, filename):
        #COMMASPACE = ', '
        # Guess the content type based on the file's extension.  Encoding
        # will be ignored, although we should check for simple things like
        # gzip'd or compressed files.
        ctype, encoding = mimetypes.guess_type(path)
        if ctype is None or encoding is not None:
            # No guess could be made, or the file is encoded (compressed), so
            # use a generic bag-of-bits type.
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        if maintype == 'text':
            fp = open(path)
            # Note: we should handle calculating the charset
            msg = MIMEText(fp.read(), _subtype=subtype)
            fp.close()
        elif maintype == 'image':
            fp = open(path, 'rb')
            msg = MIMEImage(fp.read(), _subtype=subtype)
            fp.close()
        elif maintype == 'audio':
            fp = open(path, 'rb')
            msg = MIMEAudio(fp.read(), _subtype=subtype)
            fp.close()
        else:
            fp = open(path, 'rb')
            msg = MIMEBase(maintype, subtype)
            msg.set_payload(fp.read())
            fp.close()
            # Encode the payload using Base64
            Encoders.encode_base64(msg)
        # Set the filename parameter
        msg.add_header('Content-Disposition', 'attachment', filename=filename)
        outer.attach(msg)


MAIL_SERVER='pop.free.fr'
LOGIN='bluescreen'
PASSWD='lolofer'
interpreter = MailerInterpreter(MAIL_SERVER,LOGIN,PASSWD)
interpreter.set_share_path("/home/julien/ddl/")

interpreter.dispatch_subjects()
interpreter.quit()
